<?php declare(strict_types=1);

namespace App\Services\Registry\V2;

use App\Contracts\Services\Registry\Base32;
use App\Enums\CacheTag;

final readonly class KidCommand
{
    public function __construct(
        private Base32 $base32CommandHandler,
    ) { }

    public function execute(string $privateKeyName, string $privateKeyContent): string
    {
        $seconds = 3600;

        return CacheTag::RegistryV2PrivateKey->getCache()
            ->remember(self::class . $privateKeyName, $seconds, function () use ($privateKeyContent) {
                return $this->generate($privateKeyContent);
            });
    }

    private function generate(string $privateKeyContent): string
    {
        // Extract public key from private key
        $privateKey = openssl_pkey_get_private($privateKeyContent);
        $publicKeyContent = openssl_pkey_get_details($privateKey)['key'];

        // Clean pem header and footer
        $pattern = '/-----BEGIN [^-]*-----\r?\n?|-----END [^-]*-----\r?\n?/';
        $cleanedPem = trim(preg_replace($pattern, '', $publicKeyContent));

        // Convert to der
        $der = base64_decode(preg_replace('/\s+/', '', $cleanedPem));

        // Calculate digest
        $algorithm = hash_init('sha256');
        hash_update($algorithm, $der);
        $digest = hash_final($algorithm, true);

        // Shorten digest to 30 bytes
        $digest = substr($digest, 0, 30);

        $source = $this->base32CommandHandler->encode($digest);
        $source = str_replace('=', '', $source);

        // Format with :
        $result = [];
        for ($i = 0; $i < strlen($source); $i += 4) {
            $result[] = substr($source, $i, 4);
        }

        return implode(':', $result);
    }
}
