<?php declare(strict_types=1);

namespace App\Services\Site;

use App\Dto\QuerySettingsDto;
use App\Helpers\Helpers;
use App\Models\User;
use App\Repositories\RepositoryRepository;
use App\Repositories\TagRepositoryRepository;
use App\Repositories\UserRepository;
use App\ServiceResults\ServiceResultArray;
use App\ServiceResults\ServiceResultError;
use App\ServiceResults\ServiceResultSuccess;
use App\Services\Service;
use App\Services\TagRepository\TagRepositoryCommandHandler;
use Illuminate\Support\Facades\DB;

final class TagRepositoryService extends Service
{
    public function __construct(
        private readonly RepositoryRepository        $repositoryRepository,
        private readonly UserRepository              $userRepository,
        private readonly TagRepositoryRepository     $tagRepositoryRepository,
        private readonly TagRepositoryCommandHandler $tagRepositoryCommandHandler,
    ) { }

    public function tags(string $username, string $repositoryName, QuerySettingsDto $querySettingsDto, ?User $user = null): ServiceResultArray | ServiceResultError
    {
        $pageUser = $this->userRepository->getUserByUsername($username);
        if ($pageUser === null) {
            return $this->errNotFound(__('Not Found'));
        }

        $repository = $this->repositoryRepository->getRepositoryByName($pageUser, $repositoryName);
        if (!$repository || $user === null && $repository->is_public === false) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user?->cannot('view', $repository)) {
            return $this->errFobidden(__('Access is denied'));
        }

        $tags = $this->tagRepositoryRepository->getTagsRepository(
            $repository,
            $querySettingsDto->getQueryWith()
        )->pagination(
            $querySettingsDto->getLimit(),
            $querySettingsDto->getPage()
        );

        if ($querySettingsDto->getPage() > 1 && $tags->count() === 0) {
            return $this->errNotFound(__('Not Found'));
        }

        return $this->result([
            'tags'           => $tags,
            'repository'     => $repository,
            'pageUser'       => $pageUser,
            'serviceAddress' => Helpers::dockerServiceAddress(),
        ]);
    }

    public function destroy(string $username, string $repositoryName, int $tagId, User $user): ServiceResultError | ServiceResultSuccess
    {
        $pageUser = $this->userRepository->getUserByUsername($username);
        if ($pageUser === null) {
            return $this->errNotFound(__('Not Found'));
        }

        $repository = $this->repositoryRepository->getRepositoryByName($pageUser, $repositoryName);
        if (!$repository) {
            return $this->errNotFound(__('Not Found'));
        }

        $tag = $this->tagRepositoryRepository->getTagById($tagId);
        if (!$tag || $tag->repository_id !== $repository->id) {
            return $this->errNotFound(__('Not Found'));
        }

        if ($user->cannot('delete', $tag)) {
            return $this->errFobidden(__('Access is denied'));
        }

        try {
            DB::transaction(function () use ($tag) {
                $this->tagRepositoryCommandHandler->handleDestroy($tag);
            });
        } catch (\Throwable $e) {
            \report($e);
            return $this->errService(__('Server Error'));
        }

        return $this->ok(__('site.Tag removed successfully'));
    }
}
